# Genny - xml schema generation system
# Copyright (C) 2001-2003  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#

import sys
import time
import string
import genny_util
import TextFormatter
import mismoextensions

class dtd_generator:
    """DTD Generator

    Class to transform an LDD (as represented by LDD_rows) into an XML DTD.
    
    This application was created to fill a need in the
    MISMO Servicing Workgroup.

    10/18/2001 changes to handle EXTENSION/EXTENSION_SECTION
    
    01/24/2002 changes to better handle the IDREF attribute name generation
    
    10/02/2002 changes to use the enhanced LDD in generation and general cleanup
    """
    
    def __init__(self,
                 genny_version='',
                 debug=0,
                 headerfilename='./input/bpheader.txt',
                 logfh='',
                 dtd_ext_filename='./output/MISMO-servicingtransfer-extensions.dtd',
                 LDD_rows='',
                 includecomments='NO',
                 input_name='default_input_name',
                 project_name='default_proj_name',
                 use_short_name='NO'):
        """
        Initialize the DTD generator class.
        """
        self.genny_version=genny_version
        self.genny_dtdgen_version='0.01'
        self.debug=debug
        self.headerfilename=headerfilename
        self.dtdfh = open(self.headerfilename,"w") # open the dtdfile
        self.logfh = logfh
        self.dtd_ext_filename=dtd_ext_filename
        self.dtdfh2 = open(self.dtd_ext_filename,"w") # create an empty extensions file
        self.dtdfh2.close()
        self.LDD_rows=LDD_rows
        self.includecomments=includecomments
        self.input_name=input_name
        self.project_name = project_name
        self.use_short_name = use_short_name


    def cleanup(self):
        """
        Any closing housekeeping that is necessary.
        """
        self.dtdfh.close()


    def run(self):
        """
        Perform all the steps necessary to generate the DTD.  If the class has
        been initialized properly a DTD will be  created when this method is executed.
        """
        self.header()
##        self.include_file_contents(structurefilename)
        self.containers_attr_style(self.LDD_rows,self.includecomments)
        self.cleanup()


    def header(self):
        """
        Allows for the inclusion of a DTD header file in the output DTD.
        """
        print "Adding header text to dtd ..."
        currtime = time.time() # get the current time 
        self.dtdfh.write("<?xml version='1.0' encoding='UTF-8' ?>\n\n")
        self.dtdfh.write("<!--Generated by Genny v%4s on %s                               -->\n\n" % (self.genny_version,(time.ctime(currtime))))
        bpfh = open(self.headerfilename,"r")
        bptext = bpfh.readlines()
        bpfh.close()
        self.dtdfh.writelines(bptext)
        self.logfh.write("Added boilerplate text to dtd...\n")
        self.dtdfh.write('\n<!-- declaration for a file to hold the ANY elements for extensions -->\n')
        self.dtdfh.write('<!--\n')
        self.dtdfh.write('<!ENTITY %% extension_elements SYSTEM "%s">\n' % (self.input_name+"_Extensions.dtd"))
        self.dtdfh.write('%extension_elements;\n\n')
        self.dtdfh.write(' -->\n')



    def include_file_contents(self,filename):
        """ 
        Allows addition of the contents of an arbritrary file in the the output DTD.
        """
        print "Adding file: %s to dtd ..." % filename
        self.dtdfh.write("\n")
        fh = open(filename,"r")
        filetext = fh.readlines()
        fh.close()
        self.dtdfh.writelines(filetext)
        self.logfh.write("Added %s text to dtd...\n" % (filename))


    def containers_attr_style(self,LDD_rows,commentflag):
        """ 
        Output the each of the parsed container info to the DTD file
        """
        if self.debug > 1: sys.stderr.write( "Adding containers with attribute style to dtd ...\n" )
        if self.debug > 1: sys.stderr.write( "%5i LDD Rows to process.\n" % len(LDD_rows) ) 
        row = LDD_rows[0] # extract the first row
        prev_container_name = row[2] # prime the prev name logic
        container_count = 0          # prime the container count
        containers = []
        container_rows = []
        i = 0
        while i < len(LDD_rows):
            row = LDD_rows[i]
            if prev_container_name == row[2] :
                container_rows.append(row)
                if self.debug > 1: sys.stderr.write("   same container, now appending subelement %s\n" % (row[4]) )
                self.logfh.write("   same container, now appending subelement %s\n" % (row[4]) )
            else :
                # handle end of a container
                if self.debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
                self.logfh.write("  Collected the rows , now processing %s container\n" % (prev_container_name) )
                containers.append(prev_container_name) # store the container name
                prev_container_name = row[2] # reset the prev container name
                container_count += 1
                #
                self.write_container_attr(self.dtdfh,container_rows,commentflag)
                # repeat to create/write the ele : added 02/15/2002 
                elefh= open('./projects/'+ self.project_name + '/output/ele/'+ container_rows[0][2]+ '.ele' ,"w") # open the elefile
                self.write_container_attr(elefh,container_rows,commentflag)
                elefh.close()
                #
                container_rows = [] # reset the
                container_rows.append(row)
                if self.debug > 1: sys.stderr.write("   new container, now appending subelement %s\n" % (row[4]) )
                self.logfh.write("   new container, now appending subelement %s\n" % (row[4]) )
            i += 1  # index to the next row in the LDD
            # if we've reached the end handle the last container
            #if self.debug > 1: sys.stderr.write("i is %i\n" % i )
            if i == len(LDD_rows):
                if self.debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
                self.logfh.write("  Collected the rows , now pProcessing %s container\n" % (prev_container_name) )
                container_count += 1
                self.write_container_attr(self.dtdfh,container_rows,commentflag)
                # repeat to create/write the ele : added 02/15/2002
                elefh= open('./projects/'+ self.project_name + '/output/ele/'+ container_rows[0][2]+ '.ele' ,"w") # open the elefile
                self.write_container_attr(elefh,container_rows,commentflag)
                elefh.close()
            #sys.stderr.write("%i bottom of loop\n" % i ) 
        sys.stderr.write( "%i containers and %i rows have been processed.\n" % (container_count,i) )
        self.logfh.write("%i containers and %i rows have been processed.\n" % (container_count,i))


    def write_container_attr(self,dtdfh,container_rows,commentflag):
        """
        Output a container to the DTD in the empty element style to the DTD file
        """
        row = container_rows[0]
        #
        dtdfh.write("<!-- ================================================================= -->\n")
        dtdfh.write("<!--   Container: %-50s   -->\n" % ( row[2] ))
        if len(row[1]) != 0:
            dtdfh.write("<!-- =================================================================\n" )
            # left = 0 center = 1 right = 2

            formatter = TextFormatter.TextFormatter(
                (
                    {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
                )
            )
            fout = formatter.compose(("Description: ",row[1])) 
            dtdfh.write("%s" % (fout))
            dtdfh.write("     ================================================================= -->\n" )
        else:
            dtdfh.write("<!-- ================================================================= -->\n" )
        sys.stderr.write("  writing container %s ..." % (row[2]))
        self.write_element(dtdfh,row)
        #dtdfh.write("\n")
        dtdfh.flush()
        sys.stderr.write("done.\n")                      
        
        if self.debug > 1: sys.stderr.write("    in %s there are %i rows to process\n" % (row[2],(len(container_rows)+1)))                          
        i = 0
        j = 0
        while i < len(container_rows):
            row = container_rows[i]
            # 
            # If the data item is an enumerated value then get the string
            enum_string_value = ""
            if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
                if self.debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
                enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,self.debug)
                if self.debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )
            # Output the comments if we are asked to            
            if commentflag != "NO":
                if self.debug > 1: sys.stderr.write("    adding comments %s ...\n" % (row[4]) )
                dtdfh.write("\n")
                dtdfh.write("<!-- ================================================================= -->\n")
                dtdfh.write(    "<!--   Data Item: %s -->\n" % (row[4]))

                # word wrapped description
                dtdfh.write(    "<!-- \n" )
                # left = 0 center = 1 right = 2
                formatter = TextFormatter.TextFormatter(
                    (
                        {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
                    )
                )
                dtdfh.flush()
                if self.debug > 1: sys.stderr.write("    adding xcomments %s =%s= ...\n" % (row[4],row[5]) )
                fout = formatter.compose(("Description: ",genny_util.xml_unescape(row[5]))) 
                dtdfh.write("%s" % (fout))
                dtdfh.write(" -->\n" )
    ##          dtdfh.write(    "<!-- Description: %s -->\n" % (row[5]))

                if enum_string_value == "":
                    dtdfh.write("<!--    Datatype: %s -->\n" % (row[6]))
                else:
                    dtdfh.write("<!--    Datatype: Enumerated Value List -->\n" )
                    for enum_row in enum_rows:
                        dtdfh.write("<!-- Valid Value: %s -->\n" % (enum_row[0]))
                        if len(enum_row[1]) != 0:
    ##                      dtdfh.write("Desc: %s -->\n" % (enum_row[1]))
                            # word wrapped description
                            dtdfh.write(    "<!-- \n" )
                            # left = 0 center = 1 right = 2
                            formatter = TextFormatter.TextFormatter(
                                (
                                    {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
                                )
                            )
                            fout = formatter.compose(("Value Desc: ",genny_util.xml_unescape(row[5]))) 
                            dtdfh.write("%s" % (fout))
                            dtdfh.write(" -->\n" )
                            dtdfh.write("<!-- ================================================================= -->\n")
                        else :
                            dtdfh.write("<!-- ================================================================= -->\n")
            # Output the Attribute to the DTD
            #
            # *******************************************************
            # 07-25-2003 changes to allow for the use of ShortNames
            ## pre 07-25-2003 simple setting of the attribute name
            ##attribute_name = row[4]
            if self.use_short_name != 'YES':
                attribute_name = row[4]
            else:
                attribute_name = genny_util.getShortName(row,i)
            # actually output the row
            if row[6] == "Enumerated":
                dtdfh.write("<!ATTLIST %s %s ( %s ) #IMPLIED>\n" % (row[2],attribute_name,enum_string_value) )
            # added 01/24/2002 to handle the enumerated with a default value
            elif row[6] == "EnumeratedWithDefault":
                dtdfh.write('<!ATTLIST %s %s ( %s ) "%s" >\n' % (row[2],attribute_name,enum_string_value,row[11]) )
            # added 06/26/2001 to handle the Boolean
            # a Boolean is defined as an enumerated list with Y/N as the possible values
            #
            elif row[6] == "Boolean":
                dtdfh.write("<!ATTLIST %s %s ( true | false | 0 | 1 ) #IMPLIED>\n" % (row[2],attribute_name) )
            # handle the ID datatype
            elif row[6] == "ID":
                dtdfh.write("<!ATTLIST %s %s ID  #IMPLIED>\n" % (row[2],attribute_name) )           
            # handle the IDREF datatype
            elif row[6] == "IDREF":
                ##dtdfh.write("<!ATTLIST %s %s IDREF #IMPLIED>\n" % (row[2],(row[4]+"_Referencing"+row[10]+"ID")) )
                m_refname = string.replace(row[10], "_", " ")
                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,self.logfh,"Y")
                dtdfh.write("<!ATTLIST %s %s IDREF #IMPLIED>\n" % (row[2],(attribute_name +"_Referencing_"+ m_refname + "ID")))
            # added 01/24/2002 to handle the IDREFS datatype 
            elif row[6] == "IDREFS":
                m_refname = string.replace(row[10], "_", " ")
                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,self.logfh,"Y")
                dtdfh.write("<!ATTLIST %s %s IDREFS #IMPLIED>\n" % (row[2],(attribute_name +"_Referencing_"+ m_refname + "IDs")))
            # handle the Fixed datatype
            elif row[6] == "Fixed":
                dtdfh.write("<!ATTLIST %s %s CDATA #FIXED '%s'>\n" % (row[2],attribute_name,row[11]) )
            else:
                # added 08-06-2002 to skip xmlns
                if row[4] != 'xmlns':
                    dtdfh.write("<!ATTLIST %s %s CDATA  #IMPLIED>\n" % (row[2],attribute_name) )
            # skip to the next data item we should process                  
            if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault") and j != 0:
                # skip the enumerated value rows that are passed in 
                i=i+j
                if self.debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
                j=0
            else:
                i += 1
            #dtdfh.write("\n")
        dtdfh.write("\n")


    def write_element(self,dtdfh,row):
        """
        Output a DTD element for the given container using the enhanced LDD meta data
        """
        # output the element name
        dtdfh.write('<!ELEMENT %-30s ' % (row[2]))
        # output the content
        if len(row[23]) == 0:
            # EMPTY content
            dtdfh.write(' EMPTY >\n')
        elif row[23][0][0] == 'ANY':
            # ANY content
            dtdfh.write(' ANY >\n')
        else:
            dtdfh.write('(\n')
            # otherwise this container has children containers
            ele_child_count = 0
            for child_c_list in row[23]:
                # if this is not the first child and add a comma
                ##print ("\n%s : %s : %s" % (child_c_list[0],child_c_list[1],child_c_list[2]))
                if ele_child_count != 0:
                    dtdfh.write(',\n')
                ele_cardinality = '?'  # default to 0 or 1
                # if min is zero and max is Unbounded then set to *
                if child_c_list[1] == '0' and (child_c_list[2] == 'u'or child_c_list[2] =='U'):
                    ele_cardinality = '*'
                # if both min is 1 and max is U then set to 1 or more
                elif child_c_list[1] == '1' and (child_c_list[2] == 'u' or child_c_list[2] =='U'):
                    ele_cardinality = "+"
                # otherwise if max unbounded set to zero or more    
                elif child_c_list[2] == 'u'or child_c_list[2] =='U':
                    ele_cardinality = '*'
                # if both min and max are 1 then don't have any cardinality
                # i.e. one and only one
                elif child_c_list[1] == '1' and child_c_list[1] == '1':
                    ele_cardinality = ""
                ##print ("ele cardinality %s" % (ele_cardinality))    
                # output the ELEMENT to the DTD
                dtdfh.write('               ')
                dtdfh.write('               ')
                dtdfh.write('           ')
                dtdfh.write(' %s%s' % (child_c_list[0],ele_cardinality))
                ele_child_count = ele_child_count + 1
            # close the tag
            dtdfh.write('\n')
            dtdfh.write('               ')
            dtdfh.write('               ')
            dtdfh.write('           ')
            dtdfh.write(')>\n')
