# Genny - xml schema generation system
# Copyright (C) 2001  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup

# 02/15/2002 created to test whether python template code can be created
#            for a given LDD structure
#

import sys
import time
import string
import genny_util
import TextFormatter
import mismoextensions


# **************************************************************
# Output the class header
#
def template_header(logfh,outfh,debug,__version__,classheaderfilename):
    print "Adding header text to class template ..."
    currtime = time.time() # get the current time 
    outfh.write("# --Generated by Genny v%4s on %s\n\n" % (__version__,(time.ctime(currtime))))
    bpfh = open(classheaderfilename,"r")
    bptext = bpfh.readlines()
    bpfh.close()
    outfh.writelines(bptext)
    logfh.write("Added boilerplate text to class template...\n")

# **************************************************************
# Output the non parsed part of the Class structure
#
def template_structure(logfh,outfh,debug,classstructurefilename):
    print "Adding structure text to class template ..."
    outfh.write("\n")
    fh = open(classstructurefilename,"r")
    structuretext = fh.readlines()
    fh.close()
    outfh.writelines(structuretext)
    logfh.write("Added structure text to class template...\n")


# **************************************************************
# Output/process each of the parsed container info into a python
# template
def template_containers_attr(logfh,outfh,LDD_rows,commentflag,debug,project_name):
    if debug > 1: sys.stderr.write( "Adding containers with attribute style to template ...\n" )
    if debug > 1: sys.stderr.write( "%5i LDD Rows to process.\n" % len(LDD_rows) ) 
    row = LDD_rows[0] # extract the first row
    prev_container_name = row[2] # prime the prev name logic
    container_count = 0          # prime the container count
    containers = []
    container_rows = []
    i = 0
    while i < len(LDD_rows):
        row = LDD_rows[i]
        if prev_container_name == row[2] :
            container_rows.append(row)
            if debug > 1: sys.stderr.write("   same container, now appending subelement %s\n" % (row[4]) )
            logfh.write("   same container, now appending subelement %s\n" % (row[4]) )
        else :
            # handle end of a container
            if debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
            logfh.write("  Collected the rows , now processing %s container\n" % (prev_container_name) )
            containers.append(prev_container_name) # store the container name
            prev_container_name = row[2] # reset the prev container name
            container_count += 1
            #
            outfh.write('# %s container starts in offset %i in spreadsheet\n#\n' % (container_rows[0][2],i))
            template_write_container_attr(logfh,outfh,container_rows,commentflag,debug)
            # repeat to create/write the template : added 02/15/2002 
            templfh= open('./projects/'+ project_name + '/output/templates/python/'+ container_rows[0][2]+ '.txt' ,"w") # open the template file
            template_write_container_attr(logfh,templfh,container_rows,commentflag,debug)
            templfh.close()
            #
            container_rows = [] # reset the
            container_rows.append(row)
            if debug > 1: sys.stderr.write("   new container, now appending subelement %s\n" % (row[4]) )
            logfh.write("   new container, now appending subelement %s\n" % (row[4]) )
        i += 1  # index to the next row in the LDD
        # if we've reached the end handle the last container
        #if debug > 1: sys.stderr.write("i is %i\n" % i )
        if i == len(LDD_rows):
            if debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
            logfh.write("  Collected the rows , now processing %s container\n" % (prev_container_name) )
            container_count += 1
            outfh.write('# %s container starts in offset %i in spreadsheet\n#\n' % (container_rows[0][2],i))
            template_write_container_attr(logfh,outfh,container_rows,commentflag,debug)
            # repeat to create/write the template : added 02/15/2002 
            templfh= open('./projects/'+ project_name + '/output/templates/python/'+ container_rows[0][2]+ '.txt' ,"w") # open the template file
            template_write_container_attr(logfh,templfh,container_rows,commentflag,debug)
            templfh.close()
        #sys.stderr.write("%i bottom of loop\n" % i ) 
    sys.stderr.write( "%i containers and %i rows have been processed.\n" % (container_count,i) )
    logfh.write("%i containers and %i rows have been processed.\n" % (container_count,i))


# **************************************************************
# Output a container to the DTD in the attribute style
#
def template_write_container_attr(logfh,outfh,container_rows,commentflag,debug):
    row = container_rows[0]
    element_name = row[2]
    #
    sys.stderr.write("  writing container %s to template ..." % (row[2]))
    outfh.write('    """ method for the output of the start tag for %s """\n' % (element_name))
    outfh.write('    def write_%s_start_tag(self,fh,list_of_rows):\n' % (element_name))
    outfh.write('        for row in list_of_rows:\n')
    outfh.write('            print (\n') 
    outfh.write('"""\n')
    outfh.write('< %s \n' % (element_name))
    
    if debug > 1: sys.stderr.write("    in %s there are %i rows to process\n" % (row[2],(len(container_rows)+1)))                          
    i = 0
    j = 0
    while i < len(container_rows):
        row = container_rows[i]
        # 
        # If the data item is an enumerated value then get the string
        # Note: we don't really need them now, but they may be useful in the future
        enum_string_value = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            if debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
            enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,debug)
            if debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )

        # set the default name for the attribute
        attribute_name = row[4] 
        # override the default name for the two data types that do not use the default name
        # handle the IDREF datatype
        if row[6] == "IDREF":
            m_refname = string.replace(row[10], "_", " ")
            m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,logfh,"Y")
            attribute_name = (row[4]+"_Referencing_"+ m_refname + "ID")
        # added 01/24/2002 to handle the IDREFS datatype 
        elif row[6] == "IDREFS":
            m_refname = string.replace(row[10], "_", " ")
            m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,logfh,"Y")
            attribute_name = (row[4]+"_Referencing_"+ m_refname + "IDs")

        # Output the Attribute entry to the template
        #
        outfh.write('  %s="%%s"\n' % (attribute_name))

        # skip to the next data item we should process                  
        if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault") and j != 0:
            # skip the enumerated value rows that are passed in 
            i=i+j
            if debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
            j=0
        else:
            i += 1

    # close the tag and continue 
    outfh.write('>\n')
    outfh.write('"""')
    outfh.write(' % (')
    i = 0
    while i < len(container_rows):
        if i != 0 : outfh.write(',')
        outfh.write('row[%s]' % (i))
        i += 1
    outfh.write('))\n\n')

    # output the closing tag

    outfh.write('    """ method for the output of the end tag for %s """\n' % (element_name))
    outfh.write('    def write_%s_end_tag(self,fh,list_of_rows):\n' % (element_name))    
    outfh.write('        print ( ')
    outfh.write('"""</%s>""")\n\n' % (element_name))


    sys.stderr.write("done.\n")    