# Genny - xml schema generation system
# Copyright (C) 2001  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup

# 10-13-2001 started work on xsd generation
# 08-02-2002 resumed work on the xsd generations
# 12-19-2002 fixed the annotation generation code

import sys
import time
import string
import genny_util
import TextFormatter
import mismoextensions

# **************************************************************
# Output the xsd header
#
def xsd_header(logfh,xsdfh,debug,__version__,xsd_headerfilename):
    print "Adding header text to xsd ..."
    currtime = time.time() # get the current time 
    xsdfh.write("<?xml version='1.0' encoding='UTF-8' ?>\n\n")
    xsdfh.write("<!--Generated by Genny v%4s on %s                               -->\n\n" % (__version__,(time.ctime(currtime))))
    bpfh = open(xsd_headerfilename,"r")
    bptext = bpfh.readlines()
    bpfh.close()
    xsdfh.writelines(bptext)
    logfh.write("Added boilerplate text to xsd...\n")

    
# **************************************************************
# Output the non parsed part of the DTD structure
#
def xsd_structure(logfh,xsdfh,debug,xsd_structurefilename):
    print "Adding structure text to dtd ..."
    xsdfh.write("\n")
    fh = open(xsd_structurefilename,"r")
    structuretext = fh.readlines()
    fh.close()
    xsdfh.writelines(structuretext)
    logfh.write("Added structure text to xsd...\n")


# **************************************************************
# Output/process each of the parsed container info
#
def xsd_containers(logfh,xsdfh,LDD_rows,commentflag,debug,use_short_names):
    if debug > 1: sys.stderr.write( "Adding containers with attribute style to xsd schema ...\n" )
    if debug > 1: sys.stderr.write( "%5i LDD Rows to process.\n" % len(LDD_rows) ) 
    row = LDD_rows[0] # extract the first row
    prev_container_name = row[2] # prime the prev name logic
    container_count = 0          # prime the container count
    containers = []
    container_rows = []
    i = 0
    while i < len(LDD_rows):
        row = LDD_rows[i]
        if prev_container_name == row[2] :
            container_rows.append(row)
            if debug > 1: sys.stderr.write("   same container, now appending subelement %s\n" % (row[4]) )
            logfh.write("   same container, now appending subelement %s\n" % (row[4]) )
        else :
            # handle end of a container
            if debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
            logfh.write("  Collected the rows , now processing %s container\n" % (prev_container_name) )
            containers.append(prev_container_name) # store the container name
            prev_container_name = row[2] # reset the prev container name
            container_count += 1
            #
            xsd_write_container(logfh,xsdfh,container_rows,commentflag,debug,use_short_names)
            #
            container_rows = [] # reset the
            container_rows.append(row)
            if debug > 1: sys.stderr.write("   new container, now appending subelement %s\n" % (row[4]) )
            logfh.write("   new container, now appending subelement %s\n" % (row[4]) )
        i += 1  # index to the next row in the LDD
        # if we've reached the end handle the last container
        #if debug > 1: sys.stderr.write("i is %i\n" % i )
        if i == len(LDD_rows):
            if debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
            logfh.write("  Collected the rows , now pProcessing %s container\n" % (prev_container_name) )
            container_count += 1
            xsd_write_container(logfh,xsdfh,container_rows,commentflag,debug,use_short_names)
        #sys.stderr.write("%i bottom of loop\n" % i ) 
    sys.stderr.write( "%i containers and %i rows have been processed.\n" % (container_count,i) )
    logfh.write("%i containers and %i rows have been processed.\n" % (container_count,i))



# **************************************************************
# Output a container to the schema in the attribute style
#
def xsd_write_container(logfh,xsdfh,container_rows,commentflag,debug,use_short_names):
    #
    ## commentflag = "NO" 
    row = container_rows[0]
    sys.stderr.write("  writing container %s ..." % (row[2]))
    #
    xsdfh.write("<!-- ================================================================= -->\n")
    xsdfh.write("<!--   Container: %-50s   -->\n" % ( row[2] ))
    if len(row[1]) != 0:
        xsdfh.write("<!-- =================================================================\n" )
        # left = 0 center = 1 right = 2

        formatter = TextFormatter.TextFormatter(
            (
                {'width':17, 'margin': 0, 'alignment': 2},{'width':50, 'margin': 1, 'alignment': 0}
            )
        )
        fout = formatter.compose(("Description: ",row[1])) 
        xsdfh.write("%s" % (fout))
        xsdfh.write("     ================================================================= -->\n" )
    else:
        xsdfh.write("<!-- ================================================================= -->\n" )
    # output the element definition
    # to do * output a restriction if the element is EMPTY
    #       * add the annotations
    #       * handle the ANY content model
    if len(row[23]) == 0:
        xsdfh.write('<xsd:element name="%s">\n' %(row[2]))
    elif row[23][0][0] == 'ANY':
        xsdfh.write('<xsd:element name="%s">\n' %(row[2]))
    else:
        xsdfh.write('<xsd:element name="%s">\n' %(row[2]))

    if len(row[1]) != 0 and commentflag != "NO":
        # clean-up the description
        row[1] = string.replace(row[1],">","&gt;")
        row[1] = string.replace(row[1],"<","&lt;")
        row[1] = string.replace(row[1],"&","&amp;")
        row[1] = string.replace(row[1],"\"","&quot;")       
        xsdfh.write('    <xsd:annotation>\n' )
        xsdfh.write('        <xsd:documentation xml:lang="en" >%s' % (row[1]) )
        xsdfh.write('</xsd:documentation>' )
        xsdfh.write('    </xsd:annotation>\n' )

    if len(row[23]) == 0:
        xsdfh.write(' <xsd:complexType>\n')
# Adding the attribute processing here
        xsd_write_attributes(logfh,xsdfh,container_rows,commentflag,debug,use_short_names)
        xsdfh.write(' </xsd:complexType>\n')
##      xsdfh.write('  <xsd:complexContent>\n')
##      xsdfh.write('  <xsd:restriction base="xsd:anyType" />\n')
##      xsdfh.write('  </xsd:complexContent>\n')
    elif row[23][0][0] == 'ANY':
        xsdfh.write(' <xsd:complexType mixed="true">\n')
        xsdfh.write('  <xsd:choice>\n')
        xsdfh.write('   <xsd:any namespace="##any" minOccurs="0" maxOccurs="unbounded" processContents="skip" />\n')
        xsdfh.write('  </xsd:choice>\n')        
# Adding the attribute processing here
        xsd_write_attributes(logfh,xsdfh,container_rows,commentflag,debug,use_short_names)
##      xsdfs.write('<xsd:anyAttribute namespace="##any" processContents="skip" />\n')
        xsdfh.write(' </xsd:complexType>\n')
    else:
        xsdfh.write(' <xsd:complexType>\n')
        xsdfh.write('  <xsd:sequence>\n')
        # output the the ref's to the other elements
##      sys.stderr.write("child list %s\n" % (str(row[21])))
        for child_c_list in row[23]:
            if child_c_list[2] == 'u'or child_c_list[2] =='U':
                child_c_list[2] = 'unbounded'
            # otherwise ref the defined elements 
            xsdfh.write('  <xsd:element ref = "%s" minOccurs = "%s" maxOccurs = "%s"/>\n' % (child_c_list[0],child_c_list[1],child_c_list[2]))
        xsdfh.write("  </xsd:sequence>\n")
        
##      xsdfh.write('  <xsd:attributeGroup ref="%sAttributeGroup" />\n' % (row[2]))
# Adding the attribute processing here
        xsd_write_attributes(logfh,xsdfh,container_rows,commentflag,debug,use_short_names)
        xsdfh.write(' </xsd:complexType>\n')
    xsdfh.write("</xsd:element>\n");        
    xsdfh.write("\n")
    sys.stderr.write("done.\n")


# **************************************************************
# Output all of the attributes for a given container 
#
def xsd_write_attributes(logfh,xsdfh,container_rows,commentflag,debug,use_short_names):
    debug = 0
    row = container_rows[0]
    #
    if debug > 1: sys.stderr.write("    for %s attributes there are %i rows to process\n" % (row[2],(len(container_rows)+1)))                          
    i = 0
    j = 0
    while i < len(container_rows):
        row = container_rows[i]
        # set attribute name
        attribute_name = ''
        if use_short_names != 'YES':
            attribute_name = row[4]
        else:
            attribute_name = genny_util.getShortName(row,i)
        typename = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            typename = row[4] + "EnumType"
        elif row[6] == "Boolean":
            typename = "xsd:boolean"
        # handle the ID datatype
        elif row[6] == "ID":
            typename = "xsd:ID"
        # handle the IDREF datatype
        elif row[6] == "IDREF":
            typename = "xsd:IDREF"
        # handle the IDREF datatype
        elif row[6] == "IDREFS":
            typename = "xsd:IDREFS"
        # handle the Fixed datatype
        elif row[6] == "Fixed":
            typename = "xsd:FIXED"
        else:
            typename = "xsd:string" 
        # output the documentation if requested
        if commentflag != "NO" and len(row[5]) > 0:
            # clean-up the description
            if row[6] == "Fixed":
                xsdfh.write('  <xsd:attribute name="%s" use="optional" type="xsd:string" fixed="%s" >\n' % (attribute_name,row[11]))
            elif row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
                xsdfh.write('  <xsd:attribute name="%s" use="optional" >\n' % (attribute_name))
            # added 08/06/2002 to handle the IDREF datatype
            elif row[6] == "IDREF":
                m_refname = string.replace(row[10], "_", " ")
                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,logfh,"Y")
                xsdfh.write('  <xsd:attribute name="%s" use="optional" type="%s" >\n' % ((attribute_name+"_Referencing_"+ m_refname + "ID"),typename))
            # added 08/06/2002 to handle the IDREFS datatype 
            elif row[6] == "IDREFS":
                m_refname = string.replace(row[10], "_", " ")
                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,logfh,"Y")
                xsdfh.write('  <xsd:attribute name="%s" use="optional" type="%s" >\n' % ((attribute_name+"_Referencing_"+ m_refname + "IDs"),typename))
            else:
                if row[4] != 'xmlns':
                    xsdfh.write('  <xsd:attribute name="%s" use="optional" type="%s" >\n' % (attribute_name,typename))
                    row[5] = string.replace(row[5],">","&gt;")
                    row[5] = string.replace(row[5],"<","&lt;")
                    row[5] = string.replace(row[5],"&","&amp;")
                    row[5] = string.replace(row[5],"\"","&quot;")
                    xsdfh.write('    <xsd:annotation>\n' )
                    xsdfh.write('       <xsd:documentation xml:lang="en" >%s' % (row[5]) )
                    xsdfh.write('</xsd:documentation>\n' )
                    xsdfh.write('    </xsd:annotation>\n' )

        else:
            if row[6] == "Fixed":
                xsdfh.write('  <xsd:attribute name="%s" use="optional" type="xsd:string" use="fixed" value="%s" >\n' % (row[4],row[11]))
            elif row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
                xsdfh.write('  <xsd:attribute name="%s" use="optional" >\n' % (attribute_name))
            # added 08/06/2002 to handle the IDREF datatype
            elif row[6] == "IDREF":
                m_refname = string.replace(row[10], "_", " ")
                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,logfh,"Y")
                xsdfh.write('  <xsd:attribute name="%s" use="optional" type="%s" >\n' % ((attribute_name+"_Referencing_"+ m_refname + "ID"),typename))
            # added 08/06/2002 to handle the IDREFS datatype 
            elif row[6] == "IDREFS":
                m_refname = string.replace(row[10], "_", " ")
                m_refname = mismoextensions.mismo_item_naming_scheme(m_refname,m_refname,logfh,"Y")
                xsdfh.write('  <xsd:attribute name="%s" use="optional" type="%s" >\n' % ((attribute_name+"_Referencing_"+ m_refname + "IDs"),typename))
            else:
                if row[4] != 'xmlns':
                    xsdfh.write('  <xsd:attribute name="%s" use="optional" type="%s" >\n' % (attribute_name,typename))
        # If an enum, articulate the valid values for that attribute
        # If the data item is an enumerated value then get the string
        enum_string_value = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            if debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
            enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,debug)
            enum_container_rows = collect_enum_rows(container_rows,i,j)
            xsd_write_a_enum_datatype(logfh,xsdfh,enum_container_rows,commentflag,debug)
            if debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )
        if row[4] != 'xmlns': xsdfh.write('  </xsd:attribute>\n')   
        # skip to the next data item we should process                  
        if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault") and j != 0:
            # skip the enumerated value rows that are passed in 
            i=i+j
            if debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
            j=0
        else:
            i += 1
#   xsdfh.write("\n")

#
#
def collect_enum_rows(container_rows,i,j):
    enum_container_rows = []
    row = container_rows[i]
    enum_container_rows.append(row)
    if j == 0: return enum_container_rows
    elif j == 1:
        i = i + 1
        row = container_rows[i]
        enum_container_rows.append(row)
        return enum_container_rows
    k = 1
    while k < j:
        i = i+1
        row = container_rows[i]
        enum_container_rows.append(row)
        k = k +1
    return enum_container_rows

# **************************************************************
# Output Enum for a particular attribute 
#
def xsd_write_a_enum_datatype(logfh,xsdfh,container_rows,commentflag,debug):
    #
    debug = 0
    row = container_rows[0]
    #
##  xsdfh.write("\n")
##  xsdfh.write("<!-- ================================================================= -->\n")
##  xsdfh.write("<!-- Declarations of all the Enum Datatypes                            -->\n")
##  xsdfh.write("<!-- ================================================================= -->\n")
    if debug > 1: sys.stderr.write("    in %s there are %i rows to process when generating the enum datatypes\n" % (row[2],(len(container_rows)+1)))                           
    i = 0
    j = 0
    while i < len(container_rows):
        row = container_rows[i]
        # If the data item is an enumerated value then get the string
        enum_string_value = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            if debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
            enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,debug)
            if debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )
            xsdfh.write('   <xsd:simpleType>\n' )
            if commentflag != "NO" and len(row[5]) > 0:
                # clean-up the description
                row[5] = string.replace(row[5],">","&gt;")
                row[5] = string.replace(row[5],"<","&lt;")
                row[5] = string.replace(row[5],"&","&amp;")
                row[5] = string.replace(row[5],"\"","&quot;")
                xsdfh.write('     <xsd:annotation>\n' )
                xsdfh.write('          <xsd:documentation xml:lang="en" >%s' % (row[5]) )
                xsdfh.write('</xsd:documentation>\n' )
                xsdfh.write('     </xsd:annotation>\n' )
            xsdfh.write('    <xsd:restriction base="xsd:string">\n')
            for enum_row in enum_rows:
                xsdfh.write('     <xsd:enumeration value="%s" >\n' % (enum_row[0]) )
##              sys.stderr.write("Enum Desc %s/%s\n" % (enum_row[0],enum_row[1]))
                if commentflag != "NO" and len(enum_row[1]) > 0:
                    # clean-up the description
                    enum_row[1] = string.replace(enum_row[1],">","&gt;")
                    enum_row[1] = string.replace(enum_row[1],"<","&lt;")
                    enum_row[1] = string.replace(enum_row[1],"&","&amp;")
                    enum_row[1] = string.replace(enum_row[1],"\"","&quot;")
                    xsdfh.write('     <xsd:annotation>\n' )
                    xsdfh.write('         <xsd:documentation xml:lang="en" >%s' % (enum_row[1]) )
                    xsdfh.write('</xsd:documentation>\n' )
                    xsdfh.write('     </xsd:annotation>\n' )
                xsdfh.write('     </xsd:enumeration>\n')
            xsdfh.write('    </xsd:restriction>\n')

        # skip to the next data item we should process                  
        if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault" )and j != 0:
            # skip the enumerated value rows that are passed in 
            i=i+j
            if debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
            j=0
            xsdfh.write('   </xsd:simpleType>\n')           
        else:
            i += 1
    xsdfh.write("\n")


# **************************************************************
# NO LONGER USED
# NEED TO BE UPDATED FOR SHORT NAMES IF USED AGAIN

# **************************************************************
# Output/process each of the parsed container info
#
def xsd_process_attr_groups(logfh,xsdfh,LDD_rows,commentflag,debug):
    xsdfh.write('\n')
    xsdfh.write("<!-- ================================================================= -->\n")
    xsdfh.write("<!-- Declarations of all the Attribute Groups                          -->\n")
    xsdfh.write("<!-- ================================================================= -->\n")
    
    if debug > 1: sys.stderr.write( "Adding attribute groups to xsd schema ...\n" )
    if debug > 1: sys.stderr.write( "%5i LDD Rows to process.\n" % len(LDD_rows) ) 
    row = LDD_rows[0] # extract the first row
    prev_container_name = row[2] # prime the prev name logic
    container_count = 0          # prime the container count
    containers = []
    container_rows = []
    i = 0
    while i < len(LDD_rows):
        row = LDD_rows[i]
        if prev_container_name == row[2] :
            container_rows.append(row)
            if debug > 1: sys.stderr.write("   same container, now appending subelement %s\n" % (row[4]) )
            logfh.write("   same container, now appending subelement %s\n" % (row[4]) )
        else :
            # handle end of a container
            if debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
            logfh.write("  Collected the rows , now processing %s container\n" % (prev_container_name) )
            containers.append(prev_container_name) # store the container name
            prev_container_name = row[2] # reset the prev container name
            container_count += 1
            #
            xsd_write_attr_groups(logfh,xsdfh,container_rows,commentflag,debug)
            #
            container_rows = [] # reset the
            container_rows.append(row)
            if debug > 1: sys.stderr.write("   new container, now appending subelement %s\n" % (row[4]) )
            logfh.write("   new container, now appending subelement %s\n" % (row[4]) )
        i += 1  # index to the next row in the LDD
        # if we've reached the end handle the last container
        #if debug > 1: sys.stderr.write("i is %i\n" % i )
        if i == len(LDD_rows):
            if debug > 1: sys.stderr.write( "  Collected the rows , now processing %s container\n" % prev_container_name )
            logfh.write("  Collected the rows , now pProcessing %s container\n" % (prev_container_name) )
            container_count += 1
            xsd_write_attr_groups(logfh,xsdfh,container_rows,commentflag,debug)
        #sys.stderr.write("%i bottom of loop\n" % i ) 
    sys.stderr.write( "%i containers and %i rows have been processed.\n" % (container_count,i) )
    logfh.write("%i containers and %i rows have been processed.\n" % (container_count,i))


# **************************************************************
# Output an attribute group for all the attributes for a given
# container
def xsd_write_attr_groups(logfh,xsdfh,container_rows,commentflag,debug):
    debug = 0
    ##commentflag = "YES" 
    row = container_rows[0]
    #
##  xsdfh.write("\n")
    xsdfh.write("<!-- ================================================================= -->\n")
    xsdfh.write("<!--  Attribute Group: %-35s         -->\n" % (row[2]))
    xsdfh.write("<!-- ================================================================= -->\n")
    if debug > 1: sys.stderr.write("    in %s attribute group there are %i rows to process\n" % (row[2],(len(container_rows)+1)))                          
    i = 0
    j = 0

    xsdfh.write('<xsd:attributeGroup name="%sAttributeGroup">\n' % row[2] )
    
    while i < len(container_rows):
        row = container_rows[i]
        typename = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            typename = row[4] + "EnumType"
        elif row[6] == "Boolean":
            typename = "xsd:boolean"
        # handle the ID datatype
        elif row[6] == "ID":
            typename = "xsd:ID"
        # handle the IDREF datatype
        elif row[6] == "IDREF":
            typename = "xsd:IDREF"
        # handle the Fixed datatype
        elif row[6] == "Fixed":
            typename = "xsd:FIXED"
        else:
            typename = "xsd:string" 
        # output the documentation if requested
        if commentflag != "NO" and len(row[5]) > 0:
            # clean-up the description
            if row[6] == "Fixed":
                xsdfh.write(' <xsd:attribute name="%s" use="optional" type="xsd:string" fixed="%s">\n' % (row[4],row[11]))
            else:
                if row[4] != 'xmlns':
                    xsdfh.write(' <xsd:attribute name="%s" use="optional" type="%s">\n' % (row[4],typename))
                    row[5] = string.replace(row[5],">","&gt;")
                    row[5] = string.replace(row[5],"<","&lt;")
                    row[5] = string.replace(row[5],"&","&amp;")
                    row[5] = string.replace(row[5],"\"","&quot;")
                    xsdfh.write('    <xsd:annotation>\n' )
                    xsdfh.write('        <xsd:documentation xml:lang="en" >%s' % (row[5]) )
                    xsdfh.write('</xsd:documentation>\n' )
                    xsdfh.write('    </xsd:annotation>\n' )
                    xsdfh.write(' </xsd:attribute>\n')
        else:
            if row[6] == "Fixed":
                xsdfh.write(' <xsd:attribute name="%s" use="optional" type="xsd:string" use="fixed" value="%s"/>\n' % (row[4],row[11]))
            else:
                if row[4] != 'xmlns':
                    xsdfh.write(' <xsd:attribute name="%s" use="optional" type="%s"/>\n' % (row[4],typename))
            
        # If the data item is an enumerated value then get the string
        enum_string_value = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            if debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
            enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,debug)
            if debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )
        # skip to the next data item we should process                  
        if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault") and j != 0:
            # skip the enumerated value rows that are passed in 
            i=i+j
            if debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
            j=0
        else:
            i += 1
    xsdfh.write('</xsd:attributeGroup>\n')
    xsdfh.write("\n")

# **************************************************************
# Output all of the Enum datatypes
#
def xsd_write_enum_datatypes(logfh,xsdfh,container_rows,commentflag,debug):
    #
    debug = 0
    ##commentflag = "YES" 
    row = container_rows[0]
    #
    xsdfh.write("\n")
    xsdfh.write("<!-- ================================================================= -->\n")
    xsdfh.write("<!-- Declarations of all the Enum Datatypes                            -->\n")
    xsdfh.write("<!-- ================================================================= -->\n")
    if debug > 1: sys.stderr.write("    in %s there are %i rows to process when generating the enum datatypes\n" % (row[2],(len(container_rows)+1)))                           
    i = 0
    j = 0
    while i < len(container_rows):
        row = container_rows[i]
        # If the data item is an enumerated value then get the string
        enum_string_value = ""
        if row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault":
            if debug > 1: sys.stderr.write("    getting enum value %s %i ..." %(row[4],i) )
            enum_string_value,j,enum_rows = genny_util.get_enum_string_value(i,container_rows,debug)
            if debug > 1: sys.stderr.write(" found enum str %s %i .\n" % (enum_string_value,j) )
            xsdfh.write('   <xsd:simpleType name="%sEnumType">\n' % (row[4]))
            if commentflag != "NO" and len(row[5]) > 0:
                # clean-up the description
                row[5] = string.replace(row[5],">","&gt;")
                row[5] = string.replace(row[5],"<","&lt;")
                row[5] = string.replace(row[5],"&","&amp;")
                row[5] = string.replace(row[5],"\"","&quot;")
                xsdfh.write('    <xsd:annotation>\n' )
                xsdfh.write('        <xsd:documentation xml:lang="en" >%s' % (row[5]) )
                xsdfh.write('</xsd:documentation>\n' )
                xsdfh.write('   </xsd:annotation>\n' )
            xsdfh.write('    <xsd:restriction base="xsd:string">\n')
            for enum_row in enum_rows:
                xsdfh.write('     <xsd:enumeration value="%s"/>\n' % (enum_row[0]) )
##              sys.stderr.write("Enum Desc %s/%s\n" % (enum_row[0],enum_row[1]))
                if commentflag != "NO" and len(enum_row[1]) > 0:
                    # clean-up the description
                    enum_row[1] = string.replace(enum_row[1],">","&gt;")
                    enum_row[1] = string.replace(enum_row[1],"<","&lt;")
                    enum_row[1] = string.replace(enum_row[1],"&","&amp;")
                    enum_row[1] = string.replace(enum_row[1],"\"","&quot;")
                    xsdfh.write('    <xsd:annotation>\n' )
                    xsdfh.write('        <xsd:documentation xml:lang="en" >%s' % (enum_row[1]) )
                    xsdfh.write('</xsd:documentation>\n' )
                    xsdfh.write('    </xsd:annotation>\n' )
            xsdfh.write('    </xsd:restriction>\n')

        # skip to the next data item we should process                  
        if (row[6] == "Enumerated" or row[6] == "EnumeratedWithDefault" )and j != 0:
            # skip the enumerated value rows that are passed in 
            i=i+j
            if debug > 1: sys.stderr.write("   skipping %i rows at %i.\n" % (j,i) )
            j=0
            xsdfh.write('   </xsd:simpleType>\n')           
        else:
            i += 1
    xsdfh.write("\n")
    