#
# Copyright (C) 2001-2003  James C. McDonald
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#    Free Software Foundation, Inc.
#    59 Temple Place - Suite 330
#    Boston, MA  02111-1307, USA.
#
# created by:  Jim C. McDonald
#              McDonald Computer Corp.
#
# This application was created to fill a need in the
# MISMO Servicing Workgroup
#

# MISMO extensions module
#
# 07/16/2001 created as a separate module

import ASV
import sys
import time
import string

__version__ = ".043"

# **************************************************************
# routine to implement the MISMO naming scheme per 06/29/2001
# e-mail from Ted Adams
#

# 07/16/2001 enhanced to support the AUS dtd
# 07/18/2001 added MI_FHA combination
# 07/23/2001 added ID as a valid abbreviation
# 08/06/2001 added default option to support using a fully qualified
#            name
# 08/07/2001 added SSN
# 01/03/2002 added ACH_ABA,APO,FPO,APO_FPO,REO,SFDMS
# 08/12/2002 streamlined and improved
# 02/06/2003 added OFAC,FRB,UST,WSJ,FHFB,OTS,MIN,NSF
#
def mismo_item_naming_scheme(in_name,container_name,logfh,use_full_name='N'):
    ## use_full_name='N' # temp for testing

    approved_list=['ACH','IRS','ABA','MI','FHA','VA','MERS','ARM','ID','IDREF','IDREFS',\
                   'USA','US','MI_FHA','MISMO','SSN','xmlns','ACH_ABA','APO_FPO','APO','FPO',\
                   'REO','SFDMS','UPB','LTV','APR','CUSIP','FICO','FDIC','FEMA','FHLBB','GNMA',\
                   'HMDA','IRA','LIBOR','MSA','MIP','PUD','PITI','RESPA','URLA','HUD','GSE',\
                   'HELOC','FNM','FRE','FHA_VA','FHA_MI','CAIVRS','MBS','FIPS','NFIP',\
                   'LOMA','LOMR','SMART','MIME','OFAC','FRB','UST','WSJ','FHFB','OTS',\
                   'MIN','NSF']

    #
    # apply any string conversions to the in_name first
    # these will usually be replacing a certain character
    # string with it's abbreviation and/or acronym
    #
    in_name = string.replace(in_name,"Fannie Mae","FNM")
    in_name = string.replace(in_name,"Freddie Mac","FRE")
    in_name = string.replace(in_name,"FHA MI","FHA_MI")
    in_name = string.replace(in_name,"FHA VA","FHA_VA")
    in_name = string.replace(in_name,"Credit Alert Interactive Voice Response System","CAIVRS")
    in_name = string.replace(in_name,"Mortgage Insurance","MI")
    in_name = string.replace(in_name,"MI FHA","MI_FHA")
    in_name = string.replace(in_name,"Social Security Number","SSN")
    in_name = string.replace(in_name,"ACH ABA","ACH_ABA")
    in_name = string.replace(in_name,"APO FPO","APO_FPO")
    in_name = string.replace(in_name,"IDRef","IDREF")
    in_name = string.replace(in_name,"ID Ref","IDREF")
    in_name = string.replace(in_name,"Office of Foreign Assets","OFAC")

    # 
    out_name = ""
    word_list = string.split(in_name)
    for word in word_list:
        match_word = 0
        for appr_word in approved_list:
            if word == appr_word:
                out_name = out_name + word
                match_word = 1
        if match_word == 0 :
            out_name = out_name + string.capitalize(word)
    # out_name should now be upper Camel case, no spaces

    container_out = ""
    cont_word_list = string.split(container_name)
    for cword in cont_word_list:
        match_word = 0
        for appr_word in approved_list:
            if cword == appr_word:
                container_out = container_out + cword
                match_word = 1
        if match_word == 0 :
            container_out = container_out + string.capitalize(cword)

    if use_full_name == 'N':
        container_name_len = len(container_out)
    #
    # the code below is to eliminate the container name if
    # duplicated in the tag name 
        logfh.write("container name >%s< len >%i< out name >%s< \n" % \
                (container_out,container_name_len,out_name[0:container_name_len]))  
        if container_out == out_name[0:container_name_len]:
            out_name = out_name[container_name_len:]
            out_name = "_" + out_name

    # if the name ends in ID then add an underscore on the end
    # commented out 01/24/2002 jcm
##  if out_name[-2:] == "ID":
##      out_name = out_name[:-2] + "_ID"

    # if the name is MERSID then change it to MERS_ID - quick hack
##  if out_name == "MERSID":
##      out_name = "MERS_ID"

    # fix the problem if the out_name is outnameID and out_name is
    # an abbreviation
    end_name = out_name[-2:]
    begin_name = out_name[:-2]
    if end_name == 'ID':
        match_word = 0
        for appr_word in approved_list:
            if begin_name == appr_word:
                out_name = begin_name + "_" + end_name
                match_word = 1
        if match_word == 0:
            pass # out_name is Ok as is

    return out_name